using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using SLMessageQueue2Lib;

namespace SLMessageSample
{
	public partial class Form1 : Form
	{
		private ISLMSConnection _connection = null;
		private ISLMSQueue _queue = null;
		private const string _queueNameTemplate = "FDOnAir{0}";
		private const string _subject = "OnAir1.Mirror";
		private int _messId = 0;

		/*
		 * NOTE: All Message Numbers below 0x0400 are RESERVED.
		 *
		 * Private Window Messages Start Here:
		 */
		private const int WM_USER = 0x0400;

		private const int _noifyCmd = WM_USER + 200;


		public Form1()
		{
			InitializeComponent();
			send.Enabled = false;
		}

		private void Form1_Load(object sender, EventArgs e)
		{
			try {
				_connection = (ISLMSConnection)(new SLMSConnection());
				if (string.IsNullOrEmpty(tbComputerName.Text)) {
					string computerName;
					_connection.GetName(out computerName);
					tbComputerName.Text = computerName;
				}
				if (string.IsNullOrEmpty(tbQueueName.Text)) {
					tbQueueName.Text = "TestQueue";
				}
				if (string.IsNullOrEmpty(tbQueueSend.Text)) {
					tbQueueSend.Text = string.Format(_queueNameTemplate, 1);
				}
			}
			catch (COMException ex) {
				MessageBox.Show(ex.Message, "InitError", MessageBoxButtons.OK, MessageBoxIcon.Error);
				Close();
			}
		}

		private void Form1_FormClosed(object sender, FormClosedEventArgs e)
		{
			Properties.Settings.Default.Save();
		}

		private void connect_Click(object sender, EventArgs e)
		{
			if (_queue == null) {
				Create();
			}
			else {
				Release();
			}
		}

		private void Create()
		{
			try {
				object queue;
				SLMSErrorCode code;
				int hr;
				_connection.CreateQueue(out code, out hr, tbQueueName.Text, this.Handle.ToInt32(), _noifyCmd, 0, 0, out queue);
				_queue = queue as ISLMSQueue;
				if (_queue == null) {
					string msg = string.Format("Error creating queue code:{0} HRESULT:{1} QueueName:'{2}'", code.ToString(), hr, tbQueueName.Text);
					MessageBox.Show(msg);
					return;
				}
				connect.Text = "Release";
				send.Enabled = true;
			}
			catch (COMException ex) {
				MessageBox.Show(ex.Message, "CreateQueue", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void Release() 
		{
			if (_queue != null) {
				Marshal.ReleaseComObject(_queue);
				_queue = null;
			}
			connect.Text = "Create";
			send.Enabled = false;
		}

		private void send_Click(object sender, EventArgs e)
		{
			if (_queue == null)
				return;
			try {
				SLMSErrorCode code;
				int hr;
				_queue.SendMessage(out code, out hr, _messId++, 0, cbNotify.Checked ? 1 : 0,
					tbComputerName.Text,
					tbQueueSend.Text,
					tbSubject.Text,
					tbCommand.Text);
				if (code != SLMSErrorCode.mcSuccess) {
					//   
					string msg = string.Format("Error SendMessage() code:{0} HRESULT:{1}",
						code.ToString(), hr);
					MessageBox.Show(msg);
				}
			}
			catch (COMException ex) {
				MessageBox.Show(ex.Message, "queue.SendMessage", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		protected override void DefWndProc(ref Message m)
		{
			if (m.Msg == _noifyCmd)	{
				GetMessage();
				return;
			}
			base.DefWndProc(ref m);
		}

		//      
		//       
		private void GetMessage()
		{
			try
			{
				SLMSMessageType MessType;
				SLMSErrorCode code;
				int hr;
				int MessId, ReplyTo, SentLo, SentHi;
				string FromMachine, FromQueue, Subject, Body;
				//   
				_queue.GetMessage(out code, out hr,
									out MessType, out MessId, out ReplyTo,
									out SentLo, out SentHi,
									out FromMachine, out FromQueue,
									out Subject, out Body);
				if (/*res != 0 ||*/ code != SLMSErrorCode.mcSuccess) {
					//   ,    
					return;
				}
				//   
				if (MessType == SLMSMessageType.mtError) {
					//   
					string msg = string.Format("Error GetMessage() code:{0} HRESULT:{1} FromMachine:{2} FromQueue:{3} ReplyTo:{4}", 
						code.ToString(), hr, FromMachine, FromQueue, ReplyTo);
					MessageBox.Show(msg);
				}
				else if (MessType == SLMSMessageType.mtMessage || MessType == SLMSMessageType.mtNotify) {
					//   -
					Work(FromMachine, FromQueue, Subject, Body);
				}
				else {
					//  
				}
			}
			catch (COMException ex) {
				MessageBox.Show(ex.Message, "queue.SendMessage", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void Work(string FromMachine, string FromQueue, string Subject, string Body)
		{
// 			string msg = string.Format("FromMachine:{0} FromQueue:{1} Subject:{2} Body:{3}",
// 				FromMachine, FromQueue, Subject, Body);

			string[] sub = new string[] { FromMachine, FromQueue, Subject, Body};
			ListViewItem item = new ListViewItem(sub);
			lwNotification.Items.Add(item);
		}
	}
}